import { get, kebabCase, capitalize, head, merge } from 'lodash';
import styled, { css, keyframes, ThemeProvider } from 'styled-components';
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { space, display, color, height, width, fontSize, fontWeight, textAlign, lineHeight, alignSelf, justifySelf, flex, borderRadius, themeGet, flexWrap, flexDirection, alignItems, justifyContent } from 'styled-system';
import { darken, rgba, lighten } from 'polished';
import { fadeIn, fadeOut, fadeInUp, fadeOutUp } from 'react-animations';

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var taggedTemplateLiteral = function (strings, raw) {
  return Object.freeze(Object.defineProperties(strings, {
    raw: {
      value: Object.freeze(raw)
    }
  }));
};

var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

var _templateObject = taggedTemplateLiteral(['\n    ', '\n    ', '\n  '], ['\n    ', '\n    ', '\n  ']);

var getFromTheme = function getFromTheme(theme, lookup, fallback) {
  return get(theme, lookup, fallback);
};

var getComponentVariant = function getComponentVariant(theme, key, variant) {
  var config = getFromTheme(theme, 'components.' + key + '.variants.' + variant, theme.variants[variant]);
  if (!config) throw new Error('Refractal: "' + variant + '" variant not found in theme...');
  return config;
};

var getComponentStyle = function getComponentStyle(key) {
  return function (_ref) {
    var theme = _ref.theme;
    return getFromTheme(theme, 'components.' + key + '.style', {});
  };
};

var getComponentClassName = function getComponentClassName(name) {
  return function (_ref2) {
    var classPrefix = _ref2.theme.classPrefix,
        variant = _ref2.variant;
    return (classPrefix + '-' + name + ' ' + (variant ? classPrefix + '-' + name + '-' + variant : '')).trim();
  };
};

var createComponent = function createComponent(_ref3) {
  var name = _ref3.name,
      _ref3$tag = _ref3.tag,
      tag = _ref3$tag === undefined ? 'div' : _ref3$tag,
      as = _ref3.as,
      _ref3$attrs = _ref3.attrs,
      attrs = _ref3$attrs === undefined ? {} : _ref3$attrs;

  var component = as ? styled(as) : styled[tag];

  return component.attrs(_extends({
    className: getComponentClassName(kebabCase(name))
  }, attrs))(_templateObject, getComponentStyle(name), function (_ref4) {
    var _ref4$styles = _ref4.styles,
        styles = _ref4$styles === undefined ? {} : _ref4$styles;
    return styles[name] || {};
  });
};

var _templateObject$1 = taggedTemplateLiteral(['\n  ', ';\n\n  ', ';\n'], ['\n  ', ';\n\n  ', ';\n']),
    _templateObject2 = taggedTemplateLiteral(['\n      padding: 1rem;\n      margin-bottom: 1rem;\n      border: 0;\n      font-size: 14px;\n      font-family: ', ';\n      border-radius: ', 'px;\n      background: ', ';\n      color: ', ';\n\n      a {\n        color: inherit;\n        text-decoration: underline;\n      }\n    '], ['\n      padding: 1rem;\n      margin-bottom: 1rem;\n      border: 0;\n      font-size: 14px;\n      font-family: ', ';\n      border-radius: ', 'px;\n      background: ', ';\n      color: ', ';\n\n      a {\n        color: inherit;\n        text-decoration: underline;\n      }\n    ']);

var StyledAlert = createComponent({
  name: 'Alert'
}).extend(_templateObject$1, function (_ref) {
  var variant = _ref.variant,
      theme = _ref.theme;

  var _getComponentVariant = getComponentVariant(theme, 'Alert', variant),
      backgroundColor = _getComponentVariant.backgroundColor,
      fontColor = _getComponentVariant.fontColor;

  return css(_templateObject2, theme.typography.fontFamily || 'inherit', theme.elementRadius, backgroundColor, fontColor);
}, space);

var Alert = function Alert(props) {
  return React.createElement(StyledAlert, props);
};

Alert.propTypes = {
  variant: PropTypes.string
};

Alert.defaultProps = {
  variant: 'primary'
};

var _templateObject$2 = taggedTemplateLiteral(['\n  ', '\n  ', '\n  ', '\n  ', '\n  ', '\n  ', '\n  ', '\n  ', '\n  ', '\n  ', '\n  ', '\n  ', '\n'], ['\n  ', '\n  ', '\n  ', '\n  ', '\n  ', '\n  ', '\n  ', '\n  ', '\n  ', '\n  ', '\n  ', '\n  ', '\n']);

var Box = createComponent({
  name: 'Box'
}).extend(_templateObject$2, display, space, color, fontSize, fontWeight, textAlign, lineHeight, height, width, flex, alignSelf, justifySelf);

Box.propTypes = _extends({}, display.propTypes, space.propTypes, color.propTypes, width.propTypes, fontSize.propTypes, fontWeight.propTypes, textAlign.propTypes, lineHeight.propTypes, flex.propTypes, alignSelf.propTypes, justifySelf.propTypes);

var _templateObject$3 = taggedTemplateLiteral(['\n  from {\n    transform: rotate(0deg);\n  }\n\n  to {\n    transform: rotate(360deg);\n}'], ['\n  from {\n    transform: rotate(0deg);\n  }\n\n  to {\n    transform: rotate(360deg);\n}']),
    _templateObject2$1 = taggedTemplateLiteral(['\n  color: transparent !important;\n  pointer-events: none;\n  position: relative;\n  text-index: -9999px;\n  opacity: 0.75;\n\n  &::after {\n    display: block;\n    content: \'\';\n    border-color: ', ';\n    animation: ', ' 820ms infinite linear;\n    border-width: ', 'px;\n    border-style: solid;\n    border-radius: 100%;\n    border-right-color: transparent;\n    border-top-color: transparent;\n    height: ', 'px;\n    left: 50%;\n    margin-left: -', 'px;\n    margin-top: -', 'px;\n    position: absolute;\n    top: 48%;\n    width: ', 'px;\n  }\n'], ['\n  color: transparent !important;\n  pointer-events: none;\n  position: relative;\n  text-index: -9999px;\n  opacity: 0.75;\n\n  &::after {\n    display: block;\n    content: \'\';\n    border-color: ', ';\n    animation: ', ' 820ms infinite linear;\n    border-width: ', 'px;\n    border-style: solid;\n    border-radius: 100%;\n    border-right-color: transparent;\n    border-top-color: transparent;\n    height: ', 'px;\n    left: 50%;\n    margin-left: -', 'px;\n    margin-top: -', 'px;\n    position: absolute;\n    top: 48%;\n    width: ', 'px;\n  }\n']),
    _templateObject3 = taggedTemplateLiteral(['\n  ', ';\n'], ['\n  ', ';\n']),
    _templateObject4 = taggedTemplateLiteral(['\n      font-family: inherit;\n      display: inline-block;\n      text-align: center;\n      cursor: pointer;\n      text-transform: capitalize;\n      font-weight: bold;\n      text-decoration: none;\n      appearance: none;\n      border-radius: ', 'px;\n      pointer-events: ', ';\n      opacity: ', ';\n      color: ', ';\n      height: ', 'px;\n      padding: 0 ', 'px;\n      font-size: ', 'px;\n      width: ', ';\n      background: ', ';\n      border: ', ';\n      transition: 175ms;\n\n      ', ';\n      ', ';\n\n      &:hover {\n        background: ', ';\n        border-color: ', ';\n      }\n\n      &:active {\n        background: ', ';\n        border-color: ', ';\n      }\n    '], ['\n      font-family: inherit;\n      display: inline-block;\n      text-align: center;\n      cursor: pointer;\n      text-transform: capitalize;\n      font-weight: bold;\n      text-decoration: none;\n      appearance: none;\n      border-radius: ', 'px;\n      pointer-events: ', ';\n      opacity: ', ';\n      color: ', ';\n      height: ', 'px;\n      padding: 0 ', 'px;\n      font-size: ', 'px;\n      width: ', ';\n      background: ', ';\n      border: ', ';\n      transition: 175ms;\n\n      ', ';\n      ', ';\n\n      &:hover {\n        background: ', ';\n        border-color: ', ';\n      }\n\n      &:active {\n        background: ', ';\n        border-color: ', ';\n      }\n    ']),
    _templateObject5 = taggedTemplateLiteral(['\n  & > *:not(:first-child) {\n    margin-left: 1rem;\n  }\n'], ['\n  & > *:not(:first-child) {\n    margin-left: 1rem;\n  }\n']);

var spinKeyframes = keyframes(_templateObject$3);

var loadingCss = function loadingCss(_ref) {
  var height$$1 = _ref.height,
      fontColor = _ref.fontColor;
  return css(_templateObject2$1, fontColor, spinKeyframes, height$$1 / 12, height$$1 / 2, height$$1 / 4, height$$1 / 4, height$$1 / 2);
};

var StyledButton = createComponent({
  name: 'Button',
  tag: 'button'
}).extend(_templateObject3, function (_ref2) {
  var variant = _ref2.variant,
      size = _ref2.size,
      theme = _ref2.theme,
      _ref2$outline = _ref2.outline,
      outline = _ref2$outline === undefined ? false : _ref2$outline,
      _ref2$block = _ref2.block,
      block = _ref2$block === undefined ? false : _ref2$block,
      _ref2$disabled = _ref2.disabled,
      disabled = _ref2$disabled === undefined ? false : _ref2$disabled,
      _ref2$loading = _ref2.loading,
      loading = _ref2$loading === undefined ? false : _ref2$loading,
      _ref2$transparent = _ref2.transparent,
      transparent = _ref2$transparent === undefined ? false : _ref2$transparent,
      _ref2$height = _ref2.height,
      height$$1 = _ref2$height === undefined ? theme.heights[size] : _ref2$height,
      _ref2$fontSize = _ref2.fontSize,
      fontSize$$1 = _ref2$fontSize === undefined ? theme.fontSizes[size] : _ref2$fontSize,
      _ref2$borderRadius = _ref2.borderRadius,
      borderRadius$$1 = _ref2$borderRadius === undefined ? theme.radius || 2 : _ref2$borderRadius;

  var _getComponentVariant = getComponentVariant(theme, 'Button', variant),
      backgroundColor = _getComponentVariant.backgroundColor,
      fontColor = _getComponentVariant.fontColor;

  return css(_templateObject4, borderRadius$$1, disabled ? 'none' : 'auto', disabled ? '0.65' : '1', fontColor, height$$1, height$$1 / 2, fontSize$$1, block ? '100%' : 'auto', outline || transparent ? 'transparent' : backgroundColor, transparent ? 'none' : '1px solid ' + (outline ? fontColor : darken(0.05, backgroundColor)), loading ? loadingCss({ height: height$$1, fontColor: fontColor }) : '', space, darken(0.05, backgroundColor), darken(0.05, backgroundColor), darken(0.075, backgroundColor), darken(0.075, backgroundColor));
});

var Button = function Button(props) {
  return React.createElement(StyledButton, props);
};

Button.propTypes = {
  variant: PropTypes.string,
  size: PropTypes.string,
  outline: PropTypes.bool,
  block: PropTypes.bool,
  disabled: PropTypes.bool,
  loading: PropTypes.bool,
  transparent: PropTypes.bool
};

Button.defaultProps = {
  variant: 'primary',
  size: 'md',
  outline: false,
  block: false,
  disabled: false,
  loading: false,
  transparent: false
};

Button.Group = createComponent({ name: 'ButtonGroup' }).extend(_templateObject5);

var _templateObject$4 = taggedTemplateLiteral(['\n  background: white;\n  box-shadow: ', ';\n\n  ', ';\n  ', ';\n'], ['\n  background: white;\n  box-shadow: ', ';\n\n  ', ';\n  ', ';\n']),
    _templateObject2$2 = taggedTemplateLiteral(['\n  padding: 1rem;\n  border-bottom: 1px solid ', ';\n\n  ', ';\n'], ['\n  padding: 1rem;\n  border-bottom: 1px solid ', ';\n\n  ', ';\n']),
    _templateObject3$1 = taggedTemplateLiteral(['\n  padding: 1rem;\n\n  ', ';\n'], ['\n  padding: 1rem;\n\n  ', ';\n']),
    _templateObject4$1 = taggedTemplateLiteral(['\n  padding: 1rem;\n  border-top: 1px solid ', ';\n\n  ', ';\n'], ['\n  padding: 1rem;\n  border-top: 1px solid ', ';\n\n  ', ';\n']);

var StyledCard = createComponent({
  name: 'Card'
}).extend(_templateObject$4, function (p) {
  return p.shadow ? themeGet('shadow', '0 4px 10px rgba(0, 29, 54, 0.04), 0 -2px 6px rgba(0, 29, 54, 0.025);') : 'none';
}, borderRadius, space);

var Card = function Card(props) {
  return React.createElement(StyledCard, props);
};

Card.propTypes = {
  shadow: PropTypes.bool
};

Card.defaultProps = {
  shadow: true
};

Card.Header = createComponent({
  name: 'CardHeader'
}).extend(_templateObject2$2, themeGet('colors.grayLight'), space);

Card.Body = createComponent({
  name: 'CardBody'
}).extend(_templateObject3$1, space);

Card.Footer = createComponent({
  name: 'CardFooter'
}).extend(_templateObject4$1, themeGet('colors.grayLight'), space);

var _templateObject$5 = taggedTemplateLiteral(['\n  ', '\n'], ['\n  ', '\n']),
    _templateObject2$3 = taggedTemplateLiteral(['\n      color: ', ';\n      ', ';\n    '], ['\n      color: ', ';\n      ', ';\n    ']),
    _templateObject3$2 = taggedTemplateLiteral(['\n          pointer-events: none;\n          opacity: 0.65;\n        '], ['\n          pointer-events: none;\n          opacity: 0.65;\n        ']);

var StyledIcon = createComponent({
  name: 'Icon',
  tag: 'i'
}).extend(_templateObject$5, function (p) {
  var colorFromTheme = p.theme.colors[p.color];
  var color$$1 = p.checked ? colorFromTheme || p.color : p.theme.colors.grayMid;

  return css(_templateObject2$3, color$$1, p.disabled && css(_templateObject3$2));
});

var Icon = function (_React$Component) {
  inherits(Icon, _React$Component);

  function Icon(props) {
    classCallCheck(this, Icon);

    var _this = possibleConstructorReturn(this, (Icon.__proto__ || Object.getPrototypeOf(Icon)).call(this, props));

    if (!_this.constructor.injected) {
      var link = document.createElement('link');
      link.rel = 'stylesheet';
      link.type = 'text/css';
      link.href = _this.constructor.iconUrl;
      document.head.appendChild(link);
      _this.constructor.injected = true;
    }
    return _this;
  }

  createClass(Icon, [{
    key: 'render',
    value: function render() {
      var _props = this.props,
          name = _props.name,
          size = _props.size,
          color$$1 = _props.color,
          _props$style = _props.style,
          style = _props$style === undefined ? {} : _props$style,
          _props$className = _props.className,
          className = _props$className === undefined ? '' : _props$className;
      var iconPrefix = this.constructor.iconPrefix;


      return React.createElement(StyledIcon, _extends({}, this.props, {
        className: iconPrefix + ' ' + iconPrefix + '-' + name + ' ' + className,
        style: _extends({ fontSize: size, color: color$$1 || 'inherit' }, style)
      }));
    }
  }]);
  return Icon;
}(React.Component);

Icon.injected = false;
Icon.iconUrl = 'https://cdn.heydoctor.co/assets/css/material-design-icons.css';
Icon.iconPrefix = 'mdi';


Icon.propTypes = {
  name: PropTypes.string.isRequired,
  size: PropTypes.number,
  color: PropTypes.string
};

var _templateObject$6 = taggedTemplateLiteral(['\n  display: block;\n  margin: 4px 0 0 4px;\n  color: ', ';\n  font-size: 10px;\n'], ['\n  display: block;\n  margin: 4px 0 0 4px;\n  color: ', ';\n  font-size: 10px;\n']);

var FormError = createComponent({
  name: 'FormError',
  tag: 'span'
}).extend(_templateObject$6, function (p) {
  return p.theme.colors.red;
});

var Flex = createComponent({
  name: 'Flex',
  as: Box
}).extend({ display: 'flex' }, flexWrap, flexDirection, alignItems, alignSelf, justifyContent);

Flex.displayName = 'Flex';

Flex.propTypes = _extends({}, flexWrap.propTypes, flexDirection.propTypes, alignItems.propTypes, alignSelf.propTypes, justifyContent.propTypes);

var _templateObject$7 = taggedTemplateLiteral(['\n  position: relative;\n  margin-bottom: 0;\n  cursor: pointer;\n\n  & + & {\n    margin-left: ', ';\n    margin-top: ', ';\n  }\n'], ['\n  position: relative;\n  margin-bottom: 0;\n  cursor: pointer;\n\n  & + & {\n    margin-left: ', ';\n    margin-top: ', ';\n  }\n']),
    _templateObject2$4 = taggedTemplateLiteral(['\n  display: none;\n  pointer-events: ', ';\n'], ['\n  display: none;\n  pointer-events: ', ';\n']),
    _templateObject3$3 = taggedTemplateLiteral(['\n  margin-left: 8px;\n  font-size: ', 'px;\n'], ['\n  margin-left: 8px;\n  font-size: ', 'px;\n']);

var CheckboxContainer = createComponent({
  name: 'Checkbox',
  tag: 'label'
}).extend(_templateObject$7, function (p) {
  return p.horizontal ? '12px' : 0;
}, function (p) {
  return p.horizontal ? 0 : '4px';
});

var StyledInput = createComponent({
  name: 'CheckboxInput',
  tag: 'input'
}).extend(_templateObject2$4, function (p) {
  return p.disabled ? 'none' : 'auto';
});

var StyledIcon$1 = createComponent({
  name: 'CheckboxIcon',
  as: Icon
});

var StyledLabel = createComponent({
  name: 'CheckboxLabel',
  as: Flex
}).extend(_templateObject3$3, function (p) {
  return p.fontSize;
});

var Checkbox = function (_React$Component) {
  inherits(Checkbox, _React$Component);

  function Checkbox() {
    var _ref;

    var _temp, _this, _ret;

    classCallCheck(this, Checkbox);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = Checkbox.__proto__ || Object.getPrototypeOf(Checkbox)).call.apply(_ref, [this].concat(args))), _this), _this.state = {
      value: _this.props.value
    }, _this.handleChange = function () {
      var _this$props = _this.props,
          valueTrue = _this$props.valueTrue,
          valueFalse = _this$props.valueFalse,
          onChange = _this$props.onChange;


      var newValue = _this.checked ? valueFalse : valueTrue;

      _this.setState({
        value: newValue
      }, function () {
        onChange(_this.props.name, newValue);
      });
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(Checkbox, [{
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      if (this.props.value !== undefined && this.state.value !== this.props.value) {
        this.setState({
          value: this.props.value
        });
      }
    }
  }, {
    key: 'render',
    value: function render() {
      var _props = this.props,
          label = _props.label,
          id = _props.id,
          error = _props.error,
          name = _props.name,
          fontSize$$1 = _props.fontSize,
          iconOn = _props.iconOn,
          iconOff = _props.iconOff,
          iconSize = _props.iconSize,
          color$$1 = _props.color,
          horizontal = _props.horizontal,
          disabled = _props.disabled;
      var checked = this.checked;


      return React.createElement(
        CheckboxContainer,
        { horizontal: horizontal },
        React.createElement(StyledInput, {
          id: id,
          name: name,
          type: 'checkbox',
          checked: checked,
          disabled: disabled,
          onChange: this.handleChange
        }),
        React.createElement(
          Flex,
          { alignItems: 'center' },
          React.createElement(StyledIcon$1, { size: iconSize, color: color$$1, checked: checked, name: checked ? iconOn : iconOff }),
          label && React.createElement(
            StyledLabel,
            { fontSize: fontSize$$1 },
            label
          )
        ),
        !this.state.focused && error ? React.createElement(
          FormError,
          null,
          capitalize(error)
        ) : null
      );
    }
  }, {
    key: 'checked',
    get: function get$$1() {
      return this.state.value === this.props.valueTrue;
    }
  }]);
  return Checkbox;
}(React.Component);

Checkbox.propTypes = {
  id: PropTypes.string.isRequired,
  name: PropTypes.string.isRequired,
  label: PropTypes.oneOfType([PropTypes.string, PropTypes.element]),
  value: PropTypes.oneOfType([PropTypes.number, PropTypes.string, PropTypes.bool]),
  valueTrue: PropTypes.oneOfType([PropTypes.number, PropTypes.string, PropTypes.bool]),
  valueFalse: PropTypes.oneOfType([PropTypes.number, PropTypes.string, PropTypes.bool]),
  onChange: PropTypes.func,
  iconOn: PropTypes.string,
  iconOff: PropTypes.string,
  iconSize: PropTypes.number,
  fontSize: PropTypes.number,
  color: PropTypes.string,
  horizontal: PropTypes.bool,
  disabled: PropTypes.bool
};
Checkbox.defaultProps = {
  color: 'primary',
  iconOn: 'checkbox-marked',
  iconOff: 'checkbox-blank-outline',
  valueTrue: true,
  valueFalse: false,
  iconSize: 18,
  horizontal: false,
  onChange: function onChange() {},

  disabled: false
};

var CheckboxGroup = function (_Component) {
  inherits(CheckboxGroup, _Component);

  function CheckboxGroup() {
    var _ref;

    var _temp, _this, _ret;

    classCallCheck(this, CheckboxGroup);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = CheckboxGroup.__proto__ || Object.getPrototypeOf(CheckboxGroup)).call.apply(_ref, [this].concat(args))), _this), _this.state = {
      selected: Array.isArray(_this.props.value) ? [].concat(toConsumableArray(_this.props.value)) : []
    }, _this.createChangeHandler = function (choice) {
      return function (name, value) {
        var selected = _this.state.selected;

        var newSelected = void 0;
        if (!selected.includes(value)) {
          if (choice.exclusive) {
            newSelected = [value];
          } else {
            newSelected = _this.props.choices.filter(function (c) {
              return !c.exclusive && (c.value === value || selected.includes(c.value));
            }).map(function (c) {
              return c.value;
            });
          }
        } else {
          var index = selected.indexOf(value);
          newSelected = [].concat(toConsumableArray(selected.slice(0, index)), toConsumableArray(selected.slice(index + 1)));
        }

        _this.setState({
          selected: newSelected
        }, function () {
          _this.props.onChange(_this.props.name, newSelected);
        });
      };
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(CheckboxGroup, [{
    key: 'render',
    value: function render() {
      var _this2 = this;

      var _props = this.props,
          name = _props.name,
          color$$1 = _props.color,
          choices = _props.choices,
          error = _props.error,
          horizontal = _props.horizontal;
      var selected = this.state.selected;


      return React.createElement(
        Box,
        null,
        React.createElement(
          Flex,
          { flexDirection: horizontal ? 'row' : 'column' },
          choices.length && choices.map(function (choice) {
            var _choice$value = choice.value,
                value = _choice$value === undefined ? choice.id : _choice$value;

            var key = 'checkbox-' + name + '-' + value;

            return React.createElement(Checkbox, {
              key: key,
              id: key,
              name: key,
              label: choice.label,
              color: color$$1,
              horizontal: horizontal,
              value: selected && selected.includes(value) ? value : null,
              valueTrue: value,
              valueFalse: value,
              onChange: _this2.createChangeHandler(choice)
            });
          })
        ),
        !!error && React.createElement(
          FormError,
          null,
          error
        )
      );
    }
  }]);
  return CheckboxGroup;
}(Component);

CheckboxGroup.propTypes = {
  name: PropTypes.string.isRequired,
  color: PropTypes.string,
  horizontal: PropTypes.bool,
  value: PropTypes.arrayOf(PropTypes.oneOfType([PropTypes.string, PropTypes.number])),
  choices: PropTypes.arrayOf(PropTypes.shape({
    value: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
    label: PropTypes.string,
    disabled: PropTypes.bool
  })).isRequired,
  onChange: PropTypes.func
};
CheckboxGroup.defaultProps = {
  color: 'primary',
  horizontal: false,
  onChange: function onChange() {}
};

var _templateObject$8 = taggedTemplateLiteral(['\n        display: block;\n        flex-basis: ', ';\n        max-width: ', ';\n      '], ['\n        display: block;\n        flex-basis: ', ';\n        max-width: ', ';\n      ']),
    _templateObject2$5 = taggedTemplateLiteral(['\n        max-width: 100%;\n        width: 100%;\n        flex: auto;\n      '], ['\n        max-width: 100%;\n        width: 100%;\n        flex: auto;\n      ']),
    _templateObject3$4 = taggedTemplateLiteral(['\n  margin-left: ', ';\n'], ['\n  margin-left: ', ';\n']),
    _templateObject4$2 = taggedTemplateLiteral(['\n  box-sizing: border-box;\n  flex: 1 0 auto;\n  flex-direction: column;\n  padding-left: ', 'px;\n  padding-right: ', 'px;\n\n  ', '\n'], ['\n  box-sizing: border-box;\n  flex: 1 0 auto;\n  flex-direction: column;\n  padding-left: ', 'px;\n  padding-right: ', 'px;\n\n  ', '\n']),
    _templateObject5$1 = taggedTemplateLiteral(['\n              ', ';\n              ', ';\n            '], ['\n              ', ';\n              ', ';\n            ']),
    _templateObject6 = taggedTemplateLiteral(['\n              @media (min-width: ', 'px) {\n                ', ';\n                ', ';\n              }\n            '], ['\n              @media (min-width: ', 'px) {\n                ', ';\n                ', ';\n              }\n            ']),
    _templateObject7 = taggedTemplateLiteral(['\n      order: ', ';\n      ', ';\n    '], ['\n      order: ', ';\n      ', ';\n    ']);

var getWidthPercent = function getWidthPercent(width$$1, columns) {
  return width$$1 / columns * 100 + '%';
};

var getColumnWidth = function getColumnWidth(width$$1, columns) {
  return width$$1 / columns !== 1 ? css(_templateObject$8, getWidthPercent(width$$1, columns), getWidthPercent(width$$1, columns)) : css(_templateObject2$5);
};

var getPadding = function getPadding(p) {
  if (p.collapse) return 0;
  return p.gutter ? p.gutter / 2 : p.theme.grid.gutter / 2;
};

var getOffset = function getOffset(offset, columns) {
  return css(_templateObject3$4, offset === 0 ? 0 : getWidthPercent(offset, columns));
};

var StyledColumn = createComponent({
  name: 'Column',
  as: Flex
}).extend(_templateObject4$2, getPadding, getPadding, function (p) {
  var sizes = Object.keys(p.theme.grid.sizes);
  var mediaQueries = sizes.filter(function (size) {
    return !!p[size];
  }).map(function (size) {
    return (
      // if smallest breakpoint no query needed
      size === head(sizes) ? css(_templateObject5$1, getColumnWidth(p[size], p.theme.grid.columns), p[size + 'Offset'] && getOffset(p[size + 'Offset'], p.theme.grid.columns)) : // all other breakpoints use a media query
      css(_templateObject6, p.theme.grid.sizes[size], getColumnWidth(p[size], p.theme.grid.columns), p[size + 'Offset'] !== undefined && getOffset(p[size + 'Offset'], p.theme.grid.columns))
    );
  });

  return css(_templateObject7, p.order ? p.order : 'initial', mediaQueries);
});

var Column = function Column(props) {
  return React.createElement(StyledColumn, props);
};

Column.propTypes = {
  /**
   * Margin between columns
   */
  gutter: PropTypes.number,

  /**
   * Collapse columns by removing gutters
   */
  collapse: PropTypes.bool
};

Column.defaultProps = {
  gutter: false,
  collapse: false
};

var _templateObject$9 = taggedTemplateLiteral(['\n  margin-left: auto;\n  margin-right: auto;\n  width: 100%;\n  max-width: ', ';\n  padding-left: ', 'px;\n  padding-right: ', 'px;\n'], ['\n  margin-left: auto;\n  margin-right: auto;\n  width: 100%;\n  max-width: ', ';\n  padding-left: ', 'px;\n  padding-right: ', 'px;\n']);

var getPadding$1 = function getPadding(p) {
  return p.gutter ? p.gutter / 2 : p.theme.grid.gutter / 2;
};

var StyledContainer = createComponent({
  name: 'Container',
  as: Box
}).extend(_templateObject$9, function (p) {
  return p.fluid ? '100%' : (p.maxWidth || p.theme.grid.containerMaxWidth) + 'px';
}, getPadding$1, getPadding$1);

StyledContainer.displayName = 'Container';

var Container = function Container(_ref) {
  var children = _ref.children,
      gutter = _ref.gutter,
      maxWidth = _ref.maxWidth,
      fluid = _ref.fluid;
  return React.createElement(
    StyledContainer,
    { fluid: fluid, gutter: gutter, maxWidth: maxWidth },
    React.Children.map(children, function (child) {
      return React.cloneElement(child, { gutter: gutter });
    })
  );
};

Container.propTypes = {
  gutter: PropTypes.number
};

var _templateObject$a = taggedTemplateLiteral(['\n  position: relative;\n  display: inline-block;\n'], ['\n  position: relative;\n  display: inline-block;\n']),
    _templateObject2$6 = taggedTemplateLiteral(['\n  cursor: pointer;\n'], ['\n  cursor: pointer;\n']),
    _templateObject3$5 = taggedTemplateLiteral(['\n  z-index: 10;\n  position: absolute;\n  left: 50%;\n  background: white;\n  border-radius: 2px;\n  opacity: 0;\n  visibility: hidden;\n  transform: translateX(-50%) scale(0.9);\n  transform-origin: 50% 0;\n  transition: all 300ms cubic-bezier(0.325, 0.09, 0, 1.28);\n  border: 1px solid #e4edf5;\n  box-shadow: 0 0 20px 0 rgba(178, 194, 212, 0.3);\n  min-width: 150px;\n  width: ', ';\n  ', ';\n'], ['\n  z-index: 10;\n  position: absolute;\n  left: 50%;\n  background: white;\n  border-radius: 2px;\n  opacity: 0;\n  visibility: hidden;\n  transform: translateX(-50%) scale(0.9);\n  transform-origin: 50% 0;\n  transition: all 300ms cubic-bezier(0.325, 0.09, 0, 1.28);\n  border: 1px solid #e4edf5;\n  box-shadow: 0 0 20px 0 rgba(178, 194, 212, 0.3);\n  min-width: 150px;\n  width: ', ';\n  ', ';\n']),
    _templateObject4$3 = taggedTemplateLiteral(['\n  padding: 8px 12px;\n  background: ', ';\n  font-weight: bold;\n  font-size: 12px;\n  color: ', ';\n  text-transform: uppercase;\n'], ['\n  padding: 8px 12px;\n  background: ', ';\n  font-weight: bold;\n  font-size: 12px;\n  color: ', ';\n  text-transform: uppercase;\n']),
    _templateObject5$2 = taggedTemplateLiteral(['\n  padding: 12px;\n'], ['\n  padding: 12px;\n']),
    _templateObject6$1 = taggedTemplateLiteral(['\n  display: block;\n  font-weight: 600;\n  font-size: 12px;\n  text-transform: uppercase;\n  color: ', ';\n  margin: 0 0 4px;\n'], ['\n  display: block;\n  font-weight: 600;\n  font-size: 12px;\n  text-transform: uppercase;\n  color: ', ';\n  margin: 0 0 4px;\n']),
    _templateObject7$1 = taggedTemplateLiteral(['\n  opacity: ', ';\n  pointer-events: ', ';\n  text-decoration: none;\n  color: inherit;\n  display: flex;\n  align-items: center;\n  margin-left: -12px;\n  margin-right: -12px;\n  padding: 4px 12px;\n  font-size: 14px;\n  cursor: pointer;\n\n  & + ', ' {\n    margin-top: 12px;\n  }\n\n  &:hover {\n    color: inherit;\n  }\n'], ['\n  opacity: ', ';\n  pointer-events: ', ';\n  text-decoration: none;\n  color: inherit;\n  display: flex;\n  align-items: center;\n  margin-left: -12px;\n  margin-right: -12px;\n  padding: 4px 12px;\n  font-size: 14px;\n  cursor: pointer;\n\n  & + ', ' {\n    margin-top: 12px;\n  }\n\n  &:hover {\n    color: inherit;\n  }\n']),
    _templateObject8 = taggedTemplateLiteral(['\n  background: #f9fafc;\n  padding: 12px;\n  border-radius: 0 0 7px 7px;\n  border-top: 1px solid #e9e9e9;\n  font-size: 14px;\n'], ['\n  background: #f9fafc;\n  padding: 12px;\n  border-radius: 0 0 7px 7px;\n  border-top: 1px solid #e9e9e9;\n  font-size: 14px;\n']);

var OpenStyles = {
  bl: '\n    transform: scale(1);\n  ',
  b: '\n    transform: translateX(-50%) scale(1);\n  ',
  br: '\n    transform: scale(1);\n  ',
  tl: '\n    transform: scale(1);\n  ',
  t: '\n    transform: translateX(-50%) scale(1);\n  ',
  tr: '\n    transform: scale(1);\n  '
};

var PositionStyles = {
  bl: '\n    left: 0;\n    transform: translateX(0) scale(0.9);\n    transform-origin: 0 0;\n    top: calc(100% + 8px);\n  ',
  br: '\n    left: auto;\n    right: 0;\n    top: calc(100% + 8px);\n    transform: translateX(0) scale(0.9);\n    transform-origin: 100% 0;\n  ',
  tl: '\n    left: 0;\n    bottom: calc(100% + 8px);\n    transform: translateX(0) scale(0.9);\n    transform-origin: 0 0;\n  ',
  tr: '\n    left: auto;\n    right: 0;\n    bottom: calc(100% + 8px);\n    transform: translateX(0) scale(0.9);\n    transform-origin: 100% 0;\n  '
};

var getDropdownMenuStyles = function getDropdownMenuStyles(props) {
  return '\n  ' + PositionStyles[props.position] + '\n  ' + (props.open && '\n    opacity: 1;\n    visibility: visible;\n\n    ' + OpenStyles[props.position] + ';\n  ') + '\n';
};

var DropdownProvider = styled.div(_templateObject$a);

var DropdownTrigger = styled.div(_templateObject2$6);

var DropdownMenu = styled.div(_templateObject3$5, function (p) {
  return (p.width || 150) + 'px';
}, getDropdownMenuStyles);

var Dropdown = function (_React$Component) {
  inherits(Dropdown, _React$Component);

  function Dropdown() {
    var _ref;

    var _temp, _this, _ret;

    classCallCheck(this, Dropdown);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = Dropdown.__proto__ || Object.getPrototypeOf(Dropdown)).call.apply(_ref, [this].concat(args))), _this), _this.state = {
      isOpen: false
    }, _this.toggle = function () {
      if (_this.props.on !== 'click') return;

      if (_this.state.isOpen) {
        _this.close();
      } else {
        _this.open();
      }
    }, _this.toggleHover = function () {
      if (_this.props.on !== 'hover') return;
      _this.setState({ isOpen: !_this.state.isOpen });
    }, _this.close = function () {
      _this.setState({
        isOpen: false
      }, _this.detachAutocloseListener);
    }, _this.open = function () {
      _this.setState({
        isOpen: true
      }, _this.attachAutocloseListener);
    }, _this.autoclose = function (e) {
      if (!_this.root || !_this.root.contains(e.target)) {
        _this.close();
      }
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(Dropdown, [{
    key: 'getChildContext',
    value: function getChildContext() {
      return {
        dropdown: {
          close: this.close.bind(this),
          open: this.open.bind(this)
        }
      };
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      this.detachAutocloseListener();
    }
  }, {
    key: 'attachAutocloseListener',
    value: function attachAutocloseListener() {
      if (!this.props.autoclose) {
        return;
      }
      document.addEventListener('click', this.autoclose);
    }
  }, {
    key: 'detachAutocloseListener',
    value: function detachAutocloseListener() {
      document.removeEventListener('click', this.autoclose);
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      var _props = this.props,
          position = _props.position,
          width$$1 = _props.width,
          trigger = _props.trigger,
          render = _props.render;
      var isOpen = this.state.isOpen;


      return React.createElement(
        DropdownProvider,
        {
          innerRef: function innerRef(ref) {
            _this2.root = ref;
          } },
        React.createElement(
          DropdownTrigger,
          {
            onFocus: this.toggleHover,
            onMouseOver: this.toggleHover,
            onMouseLeave: this.toggleHover,
            onClick: this.toggle,
            'aria-haspopup': 'true',
            'aria-expanded': isOpen },
          trigger
        ),
        React.createElement(
          DropdownMenu,
          { open: isOpen, position: position, width: width$$1 },
          render({
            close: this.close
          })
        )
      );
    }
  }]);
  return Dropdown;
}(React.Component);

Dropdown.propTypes = {
  trigger: PropTypes.element,
  render: PropTypes.func,
  autoclose: PropTypes.bool,
  position: PropTypes.string,
  on: PropTypes.string
};
Dropdown.defaultProps = {
  autoclose: true,
  position: 'bl',
  on: 'click'
};
Dropdown.childContextTypes = {
  dropdown: PropTypes.object
};


Dropdown.Header = styled.div(_templateObject4$3, function (p) {
  return p.theme.colors.grayLight;
}, function (p) {
  return p.theme.colors.grayMid;
});

Dropdown.Body = styled.div(_templateObject5$2);

Dropdown.Title = styled.h3(_templateObject6$1, function (p) {
  return p.theme.colors.primary;
});

Dropdown.Item = styled.div(_templateObject7$1, function (p) {
  return p.disabled ? 0.75 : 1;
}, function (p) {
  return p.disabled ? 'none' : 'initial';
}, Dropdown.Title);

Dropdown.Footer = styled.footer(_templateObject8);

var _templateObject$b = taggedTemplateLiteral(['\n  position: relative;\n  transition: 175ms;\n\n  & + & {\n    margin-top: 1rem;\n  }\n'], ['\n  position: relative;\n  transition: 175ms;\n\n  & + & {\n    margin-top: 1rem;\n  }\n']);

var Field = createComponent({
  name: 'Field'
}).extend(_templateObject$b);

var VALIDATIONS = {
  required: function required(val) {
    if (!val || typeof val === 'string' && val === '') {
      throw new Error('This field is required');
    }
  },
  minLength: function minLength(val, _minLength) {
    if (!val || ('' + val).length < _minLength) {
      throw new Error('This field must be longer than ' + _minLength + ' characters');
    }
  },
  maxLength: function maxLength(val, _maxLength) {
    if (val && ('' + val).length > _maxLength) {
      throw new Error('This field cannot be less than ' + _maxLength + ' characters');
    }
  }
};

var Formbot = function (_React$Component) {
  inherits(Formbot, _React$Component);

  function Formbot() {
    var _ref;

    var _temp, _this, _ret;

    classCallCheck(this, Formbot);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = Formbot.__proto__ || Object.getPrototypeOf(Formbot)).call.apply(_ref, [this].concat(args))), _this), _this.state = {
      values: _this.props.initialValues || {},
      fields: {},
      errors: {}
    }, _this.reset = function () {
      _this.setState({
        values: {},
        fields: {},
        errors: {}
      });
    }, _this.makeValidation = function (validation, value) {
      if (!validation.validator(value)) {
        throw new Error(validation.message || 'This field is required');
      }
    }, _this.onFocus = function (field) {
      _this.updateField(field, { focused: true }).then(function () {
        _this.props.onFocus(field);
      });
    }, _this.onChange = function (field, value) {
      _this.updateField(field, { validated: false }).then(function () {
        _this.setState({
          values: _extends({}, _this.state.values, defineProperty({}, field, value))
        }, function () {
          _this.validateField(field);
          _this.props.onChange(field, value, _this.state.values);
        });
      });
    }, _this.onBlur = function (field) {
      _this.updateField(field, { blurred: true }).then(function () {
        return _this.validateField(field);
      }).then(function () {
        _this.props.onBlur(field);
      });
    }, _this.onSubmit = function (event) {
      event.preventDefault();

      _this.validateAllFields().then(function () {
        _this.props.onSubmit({
          isValid: _this.isValid,
          values: _this.state.values,
          errors: _this.state.errors
        });
      });
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(Formbot, [{
    key: 'getValues',
    value: function getValues() {
      var _this2 = this;

      return this.validateAllFields().then(function () {
        return {
          isValid: _this2.isValid,
          values: _this2.values
        };
      });
    }
  }, {
    key: 'setValues',
    value: function setValues() {
      var values = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      this.setState({
        values: _extends({}, this.state.values, values)
      });
    }
  }, {
    key: 'updateField',
    value: function updateField(field) {
      var _this3 = this;

      var updates = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      return new Promise(function (resolve) {
        var fieldState = _this3.state.fields[field] || {};

        _this3.setState({
          fields: _extends({}, _this3.state.fields, defineProperty({}, field, _extends({}, fieldState, updates)))
        }, resolve);
      });
    }
  }, {
    key: 'validateField',
    value: function validateField(field) {
      var _this4 = this;

      return new Promise(function (resolve) {
        var validation = _this4.props.validations[field];

        if (!validation) return;

        var fieldValue = _this4.state.values[field];
        var errorMsg = void 0;

        try {
          if (typeof validation === 'function') {
            validation(fieldValue);
          } else {
            Object.keys(validation).forEach(function (method) {
              var validator = VALIDATIONS[method];

              if (!validator) {
                throw new Error('Formbot: "' + method + '" is not a built-in validator.');
              }

              validator(fieldValue, validation[method]);
            });
          }
        } catch (err) {
          errorMsg = err.message;
        } finally {
          _this4.updateField(field, { validated: true }).then(function () {
            _this4.setState({
              errors: _extends({}, _this4.state.errors, defineProperty({}, field, errorMsg))
            }, resolve);
          });
        }
      });
    }
  }, {
    key: 'validateAllFields',
    value: function validateAllFields() {
      var _this5 = this;

      return Promise.all(this.validatable.map(function (field) {
        return _this5.updateField(field, {}).then(function () {
          return _this5.validateField(field);
        });
      }));
    }
  }, {
    key: 'render',
    value: function render() {
      return this.props.children(_extends({}, this.props, {
        values: this.state.values,
        errors: this.state.errors,
        onChange: this.onChange,
        onFocus: this.onFocus,
        onBlur: this.onBlur,
        onSubmit: this.onSubmit,
        reset: this.reset
      }));
    }
  }, {
    key: 'validatable',
    get: function get$$1() {
      return Object.keys(this.props.validations);
    }
  }, {
    key: 'isValid',
    get: function get$$1() {
      return Object.values(this.state.errors).every(function (val) {
        return !val;
      });
    }
  }, {
    key: 'values',
    get: function get$$1() {
      return this.state.values;
    }
  }]);
  return Formbot;
}(React.Component);

Formbot.propTypes = {
  initialValues: PropTypes.shape(),
  validations: PropTypes.shape({
    fieldName: PropTypes.oneOfType([PropTypes.func, PropTypes.shape()])
  }),
  onFocus: PropTypes.func,
  onChange: PropTypes.func,
  onBlur: PropTypes.func,
  onSubmit: PropTypes.func
};
Formbot.defaultProps = {
  initialValues: {},
  validations: {},
  onFocus: function onFocus() {},
  onChange: function onChange() {},
  onBlur: function onBlur() {},
  onSubmit: function onSubmit() {}
};

var FormGroup = function FormGroup(_ref) {
  var children = _ref.children,
      _ref$padding = _ref.padding,
      padding = _ref$padding === undefined ? 3 : _ref$padding;
  return React.createElement(
    Box,
    { mb: padding },
    children
  );
};

var _templateObject$c = taggedTemplateLiteral(['\n  display: block;\n  transition: 250ms;\n  font-weight: 500;\n  margin: 0 0 4px 4px;\n  z-index: 10;\n  font-size: ', 'px;\n\n  ', ';\n'], ['\n  display: block;\n  transition: 250ms;\n  font-weight: 500;\n  margin: 0 0 4px 4px;\n  z-index: 10;\n  font-size: ', 'px;\n\n  ', ';\n']),
    _templateObject2$7 = taggedTemplateLiteral(['\n      position: absolute;\n      top: 2px;\n      left: 8px;\n      opacity: ', ';\n      margin: 0;\n      font-size: ', 'px;\n    '], ['\n      position: absolute;\n      top: 2px;\n      left: 8px;\n      opacity: ', ';\n      margin: 0;\n      font-size: ', 'px;\n    ']);

var Label = createComponent({
  name: 'Label',
  tag: 'label'
}).extend(_templateObject$c, function (p) {
  return p.theme.fontSizes[p.size];
}, function (props) {
  return props.isFloatable && css(_templateObject2$7, props.isFloating ? 1 : 0, props.theme.fontSizes[props.size] * 0.8);
});

var _templateObject$d = taggedTemplateLiteral(['\n  position: relative;\n'], ['\n  position: relative;\n']),
    _templateObject2$8 = taggedTemplateLiteral(['\n  ', ';\n'], ['\n  ', ';\n']),
    _templateObject3$6 = taggedTemplateLiteral(['\n    border: 1px solid ', ';\n    height: ', 'px;\n    display: block;\n    outline: none;\n    width: 100%;\n    padding: 8px;\n    border-radius: ', 'px;\n    transition: 250ms all;\n    -webkit-appearance: none;\n    font-family: inherit;\n    font-size: ', 'px;\n\n    &:hover,\n    &:focus,\n    &:active {\n      border-color: ', ';\n    }\n\n    ::placeholder {\n      color: ', ';\n    }\n\n    &[disabled] {\n      opacity: 0.65;\n    }\n\n    ', ';\n  '], ['\n    border: 1px solid ', ';\n    height: ', 'px;\n    display: block;\n    outline: none;\n    width: 100%;\n    padding: 8px;\n    border-radius: ', 'px;\n    transition: 250ms all;\n    -webkit-appearance: none;\n    font-family: inherit;\n    font-size: ', 'px;\n\n    &:hover,\n    &:focus,\n    &:active {\n      border-color: ', ';\n    }\n\n    ::placeholder {\n      color: ', ';\n    }\n\n    &[disabled] {\n      opacity: 0.65;\n    }\n\n    ', ';\n  ']),
    _templateObject4$4 = taggedTemplateLiteral(['\n        padding-bottom: 0;\n      '], ['\n        padding-bottom: 0;\n      ']);

var InputContainer = createComponent({
  name: 'InputContainer'
}).extend(_templateObject$d);

var StyledInput$1 = createComponent({
  name: 'Input',
  tag: 'input'
}).extend(_templateObject2$8, function (_ref) {
  var isFloating = _ref.isFloating,
      size = _ref.size,
      theme = _ref.theme,
      _ref$borderRadius = _ref.borderRadius,
      borderRadius$$1 = _ref$borderRadius === undefined ? theme.radius : _ref$borderRadius;
  return css(_templateObject3$6, theme.colors.grayLight, theme.heights[size], borderRadius$$1, theme.fontSizes[size], theme.colors.grayMid, theme.colors.grayMid, isFloating && css(_templateObject4$4));
});

var StyledTextArea = StyledInput$1.withComponent('textarea');

var Input = function (_React$Component) {
  inherits(Input, _React$Component);

  function Input(props) {
    classCallCheck(this, Input);

    var _this = possibleConstructorReturn(this, (Input.__proto__ || Object.getPrototypeOf(Input)).call(this, props));

    _this.getInput = function () {
      return _this.input;
    };

    _this.onFocus = function (e) {
      _this.setState({ focused: true });
      _this.props.onFocus(e.target.name);
    };

    _this.onBlur = function (e) {
      _this.setState({ focused: false });

      var _this$props = _this.props,
          onBlur = _this$props.onBlur,
          onChange = _this$props.onChange,
          transformOnBlur = _this$props.transformOnBlur;


      onBlur(e.target.name);

      var currentValue = _this.props.value;
      if (transformOnBlur && currentValue) {
        var transformedValue = transformOnBlur(currentValue);
        if (transformedValue !== currentValue) {
          onChange(e.target.name, transformedValue);
        }
      }
    };

    _this.onChange = function (e) {
      _this.autogrow();
      _this.setState({ value: e.target.value });
      _this.props.onChange(e.target.name, e.target.value);
    };

    _this.input = React.createRef();
    _this.state = {
      value: props.value || '',
      focused: false
    };
    return _this;
  }

  createClass(Input, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      if (this.props.autogrow) {
        this.createShadowElement();
        this.autogrow();
      }

      if (this.props.autofocus) {
        if (this.input.current) {
          this.input.current.focus();
        }
      }
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(props) {
      if (props.value !== this.props.value) {
        this.setState({ value: props.value });
      }
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate(prevProps) {
      if (prevProps.value !== this.props.value) {
        this.autogrow();
      }
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      if (this.shadow) {
        this.shadow.remove();
      }
    }
  }, {
    key: 'createShadowElement',
    value: function createShadowElement() {
      this.shadow = document.createElement('textarea');
      this.shadow.style.position = 'absolute';
      this.shadow.style.left = '-9000px';
      document.body.append(this.shadow);
    }
  }, {
    key: 'autogrow',
    value: function autogrow() {
      var _props = this.props,
          multiline = _props.multiline,
          autogrow = _props.autogrow,
          minRows = _props.minRows,
          maxRows = _props.maxRows,
          rowHeight = _props.rowHeight;

      if (!multiline || !autogrow) {
        return;
      }

      var minHeight = minRows * rowHeight;
      var maxHeight = maxRows * rowHeight;

      this.shadow.style.width = this.input.clientWidth + 'px';
      this.shadow.value = this.props.value || this.state.value;
      var height$$1 = this.shadow.scrollHeight + 14;
      if (height$$1 < minHeight) {
        height$$1 = minHeight;
      } else if (height$$1 > maxHeight) {
        height$$1 = maxHeight;
      }

      this.setState({ height: height$$1 });
    }
  }, {
    key: 'focus',
    value: function focus() {
      this.input.current.focus();
    }
  }, {
    key: 'blur',
    value: function blur() {
      this.input.current.blur();
    }
  }, {
    key: 'render',
    value: function render() {
      var _props2 = this.props,
          className = _props2.className,
          style = _props2.style,
          minRows = _props2.minRows,
          maxRows = _props2.maxRows,
          rowHeight = _props2.rowHeight,
          label = _props2.label,
          multiline = _props2.multiline,
          autogrow = _props2.autogrow,
          autofocus = _props2.autofocus,
          id = _props2.id,
          error = _props2.error,
          value = _props2.value,
          floating = _props2.floating,
          placeholder = _props2.placeholder,
          transformOnBlur = _props2.transformOnBlur,
          size = _props2.size,
          rest = objectWithoutProperties(_props2, ['className', 'style', 'minRows', 'maxRows', 'rowHeight', 'label', 'multiline', 'autogrow', 'autofocus', 'id', 'error', 'value', 'floating', 'placeholder', 'transformOnBlur', 'size']);


      var isFloating = floating && value && value.length > 0 || floating && this.state.value;

      var inputProps = _extends({}, rest, {
        id: id,
        innerRef: this.input,
        size: size,
        value: this.state.value,
        onChange: this.onChange,
        onFocus: this.onFocus,
        onBlur: this.onBlur,
        style: autogrow ? _extends({}, style, { height: this.state.height }) : style,
        placeholder: placeholder,
        isFloatable: floating,
        isFloating: isFloating
      });

      var Label$$1 = React.createElement(
        Label,
        { htmlFor: id, styles: rest.styles, size: size, isFloatable: floating, isFloating: isFloating },
        label
      );

      return React.createElement(
        Field,
        { styles: rest.styles },
        !floating && Label$$1,
        React.createElement(
          InputContainer,
          { styles: rest.styles },
          floating && Label$$1,
          multiline ? React.createElement(StyledTextArea, inputProps) : React.createElement(StyledInput$1, inputProps)
        ),
        !this.state.focused && error ? React.createElement(
          FormError,
          { styles: rest.styles },
          error
        ) : null
      );
    }
  }]);
  return Input;
}(React.Component);

Input.propTypes = {
  value: PropTypes.string,
  disabled: PropTypes.bool,
  placeholder: PropTypes.string,
  multiline: PropTypes.bool,
  label: PropTypes.oneOfType([PropTypes.string, PropTypes.element]),
  onFocus: PropTypes.func,
  onBlur: PropTypes.func,
  onChange: PropTypes.func,
  type: PropTypes.string,
  minRows: PropTypes.number,
  rows: PropTypes.number,
  maxRows: PropTypes.number,
  rowHeight: PropTypes.number,
  autogrow: PropTypes.bool,
  size: PropTypes.string,
  floating: PropTypes.bool
};
Input.defaultProps = {
  type: 'text',
  multiline: false,
  minRows: 2,
  rows: 3,
  maxRows: 6,
  rowHeight: 14 * 1.5,
  autogrow: false,
  disabled: false,
  size: 'md',
  onFocus: function onFocus() {},
  onBlur: function onBlur() {},
  onChange: function onChange() {},

  floating: false
};

var _templateObject$e = taggedTemplateLiteral(['\n  a {\n    color: inherit;\n    text-decoration: underline;\n  }\n'], ['\n  a {\n    color: inherit;\n    text-decoration: underline;\n  }\n']);

var Container$1 = styled.span(_templateObject$e);

var REGEX = /(\b(https?|ftp|file):\/\/[-A-Z0-9+&@#\/%?=~_|!:,.;]*[-A-Z0-9+&@#\/%=~_|])/gi;

var parse = function parse(string) {
  return string.replace(REGEX, function (url) {
    return '<a target="_blank" href="' + url + '">' + url + '</a>';
  });
};

var Linkify = function Linkify(props) {
  return React.createElement(Container$1, { style: props.style || {}, dangerouslySetInnerHTML: { __html: parse(props.children) } });
};

var _templateObject$f = taggedTemplateLiteral(['', ''], ['', '']),
    _templateObject2$9 = taggedTemplateLiteral(['\n  top: 0;\n  left: 0;\n  right: 0;\n  bottom: 0;\n  z-index: 1000;\n  padding: 12px;\n  display: flex;\n  align-items: center;\n  position: fixed;\n  overflow-y: auto;\n  overflow-x: hidden;\n  background: rgba(0, 0, 0, 0.2);\n  justify-content: center;\n  ', ';\n  ', ';\n'], ['\n  top: 0;\n  left: 0;\n  right: 0;\n  bottom: 0;\n  z-index: 1000;\n  padding: 12px;\n  display: flex;\n  align-items: center;\n  position: fixed;\n  overflow-y: auto;\n  overflow-x: hidden;\n  background: rgba(0, 0, 0, 0.2);\n  justify-content: center;\n  ', ';\n  ', ';\n']),
    _templateObject3$7 = taggedTemplateLiteral(['\n      animation: 0.35s ', ';\n    '], ['\n      animation: 0.35s ', ';\n    ']),
    _templateObject4$5 = taggedTemplateLiteral(['\n  position: relative;\n  margin: auto;\n  min-width: ', 'px;\n  max-width: ', 'px;\n  background: #ffffff;\n  background-clip: padding-box;\n  box-shadow: 0 8px 30px rgba(0, 29, 54, 0.1);\n  border-radius: 2px;\n  ', ' ', ';\n'], ['\n  position: relative;\n  margin: auto;\n  min-width: ', 'px;\n  max-width: ', 'px;\n  background: #ffffff;\n  background-clip: padding-box;\n  box-shadow: 0 8px 30px rgba(0, 29, 54, 0.1);\n  border-radius: 2px;\n  ', ' ', ';\n']),
    _templateObject5$3 = taggedTemplateLiteral(['\n  font-size: 20px;\n  font-weight: 600;\n  padding: 0 24px;\n  margin: 24px 0 0;\n'], ['\n  font-size: 20px;\n  font-weight: 600;\n  padding: 0 24px;\n  margin: 24px 0 0;\n']),
    _templateObject6$2 = taggedTemplateLiteral(['\n  padding: 24px;\n'], ['\n  padding: 24px;\n']),
    _templateObject7$2 = taggedTemplateLiteral(['\n  padding: 0 24px 24px;\n'], ['\n  padding: 0 24px 24px;\n']);

var fadeInAnimation = keyframes(_templateObject$f, fadeIn);
var fadeOutAnimation = keyframes(_templateObject$f, fadeOut);
var fadeInUpAnimation = keyframes(_templateObject$f, fadeInUp);
var fadeOutUpAnimation = keyframes(_templateObject$f, fadeOutUp);

var Backdrop = createComponent({
  name: 'ModalBackdrop'
}).extend(_templateObject2$9, function (_ref) {
  var opening = _ref.opening;
  return opening && css(_templateObject3$7, fadeInAnimation);
}, function (_ref2) {
  var closing = _ref2.closing;
  return closing && css(_templateObject3$7, fadeOutAnimation);
});

var ModalContent = createComponent({
  name: 'ModalContent'
}).extend(_templateObject4$5, function (p) {
  return p.minWidth || 250;
}, function (p) {
  return p.maxWidth || 768;
}, function (_ref3) {
  var opening = _ref3.opening;
  return opening && 'animation: 0.75s ' + fadeInUpAnimation + ';';
}, function (_ref4) {
  var closing = _ref4.closing;
  return closing && 'animation: 0.75s ' + fadeOutUpAnimation + ';';
});

var Modal = function (_React$Component) {
  inherits(Modal, _React$Component);

  function Modal(props) {
    classCallCheck(this, Modal);

    var _this = possibleConstructorReturn(this, (Modal.__proto__ || Object.getPrototypeOf(Modal)).call(this, props));

    _this.handleEscapeKey = function (event) {
      if (!_this.props.closeOnEscape) {
        return;
      }

      if (event.keyCode === 27) {
        _this.closeInternal();
      }
    };

    _this.handleBackdropClick = function () {
      if (!_this.props.closeOnBackdropClick) {
        return;
      }

      _this.closeInternal();
    };

    _this.handleContentClick = function (event) {
      event.stopPropagation();
    };

    _this.state = {
      open: props.open || false,
      opening: false,
      closing: false
    };
    return _this;
  }

  createClass(Modal, [{
    key: 'componentDidUpdate',
    value: function componentDidUpdate(oldProps) {
      if (!oldProps.open && this.props.open) {
        this.open();
      } else if (oldProps.open && !this.props.open) {
        this.close();
      }
    }
  }, {
    key: 'open',
    value: function open() {
      var _this2 = this;

      this.setState({
        opening: true
      }, function () {
        setTimeout(function () {
          document.addEventListener('keydown', _this2.handleEscapeKey);

          _this2.setState({
            open: true,
            opening: false
          });
        }, 250);
      });
    }
  }, {
    key: 'close',
    value: function close() {
      var _this3 = this;

      this.setState({
        closing: true
      }, function () {
        setTimeout(function () {
          document.removeEventListener('keydown', _this3.handleEscapeKey);

          _this3.setState({
            open: false,
            closing: false
          });
        }, 250);
      });
    }
  }, {
    key: 'closeInternal',
    value: function closeInternal() {
      if (this.isControlled) {
        this.props.onClose();
      } else {
        this.close();
      }
    }
  }, {
    key: 'render',
    value: function render() {
      if (this.isClosed) {
        return null;
      }

      var _state = this.state,
          opening = _state.opening,
          closing = _state.closing;
      var _props = this.props,
          children = _props.children,
          minWidth = _props.minWidth,
          maxWidth = _props.maxWidth;


      return React.createElement(
        Backdrop,
        { opening: opening, closing: closing, onClick: this.handleBackdropClick },
        React.createElement(
          ModalContent,
          {
            opening: opening,
            closing: closing,
            onClick: this.handleContentClick,
            minWidth: minWidth,
            maxWidth: maxWidth },
          children
        )
      );
    }
  }, {
    key: 'isOpen',
    get: function get$$1() {
      return !this.closed;
    }
  }, {
    key: 'isClosed',
    get: function get$$1() {
      var _state2 = this.state,
          open = _state2.open,
          opening = _state2.opening,
          closing = _state2.closing;

      return !open && !opening && !closing;
    }
  }, {
    key: 'isControlled',
    get: function get$$1() {
      return 'open' in this.props;
    }
  }]);
  return Modal;
}(React.Component);

Modal.propTypes = {
  open: PropTypes.bool,
  closeOnBackdropClick: PropTypes.bool,
  closeOnEscape: PropTypes.bool,
  onClose: PropTypes.func
};
Modal.defaultProps = {
  open: false,
  closeOnBackdropClick: true,
  closeOnEscape: true,
  onClose: function onClose() {}
};


Modal.Title = createComponent({
  name: 'ModalTitle',
  tag: 'h2'
}).extend(_templateObject5$3);

Modal.Body = createComponent({
  name: 'ModalBody'
}).extend(_templateObject6$2);

Modal.Footer = createComponent({
  name: 'ModalFooter'
}).extend(_templateObject7$2);

var _templateObject$g = taggedTemplateLiteral(['\n  from { transform: rotate(0deg); }\n  from { transform: rotate(360deg); }\n'], ['\n  from { transform: rotate(0deg); }\n  from { transform: rotate(360deg); }\n']),
    _templateObject2$a = taggedTemplateLiteral(['\n  ', ';\n\n  border-right-color: ', ';\n  border-radius: 50%;\n  animation: ', ' 1.1s infinite linear;\n'], ['\n  ', ';\n\n  border-right-color: ', ';\n  border-radius: 50%;\n  animation: ', ' 1.1s infinite linear;\n']),
    _templateObject3$8 = taggedTemplateLiteral(['\n    height: ', 'px;\n    width: ', 'px;\n    border: ', 'px solid ', ';\n  '], ['\n    height: ', 'px;\n    width: ', 'px;\n    border: ', 'px solid ', ';\n  ']);

var spin = keyframes(_templateObject$g);

var Spinner = createComponent({
  name: 'Spinner'
}).extend(_templateObject2$a, function (_ref) {
  var _ref$size = _ref.size,
      size = _ref$size === undefined ? 15 : _ref$size;
  return css(_templateObject3$8, size, size, size / 10, function (p) {
    return rgba(p.theme.colors.primary, 0.3);
  });
}, function (p) {
  return p.theme.colors.primary;
}, spin);

var _templateObject$h = taggedTemplateLiteral(['\n  display: flex;\n  flex-direction: column;\n  height: 100%;\n  width: 100%;\n  align-items: center;\n  justify-content: center;\n  min-height: 50px;\n'], ['\n  display: flex;\n  flex-direction: column;\n  height: 100%;\n  width: 100%;\n  align-items: center;\n  justify-content: center;\n  min-height: 50px;\n']);

var Container$2 = createComponent({
  name: 'Placeholder'
}).extend(_templateObject$h);

var Loadable = function (_React$Component) {
  inherits(Loadable, _React$Component);

  function Loadable() {
    var _ref;

    var _temp, _this, _ret;

    classCallCheck(this, Loadable);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = Loadable.__proto__ || Object.getPrototypeOf(Loadable)).call.apply(_ref, [this].concat(args))), _this), _this.state = {
      delayed: false
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(Loadable, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.runDelay();
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      clearTimeout(this.delayTimer);
    }
  }, {
    key: 'runDelay',
    value: function runDelay() {
      var _this2 = this;

      if (this.props.delay <= 0) return;

      this.setState({ delayed: true }, function () {
        _this2.delayTimer = setTimeout(function () {
          _this2.setState({
            delayed: false
          });
        }, _this2.props.delay);
      });
    }
  }, {
    key: 'render',
    value: function render() {
      var _props = this.props,
          children = _props.children,
          loading = _props.loading,
          error = _props.error,
          renderLoading = _props.renderLoading,
          renderError = _props.renderError,
          onReload = _props.onReload;


      if (loading && this.state.delayed) {
        return null;
      }

      if (loading) {
        return renderLoading ? renderLoading() : React.createElement(
          Container$2,
          null,
          React.createElement(Spinner, null)
        );
      }

      if (error) {
        return renderError ? renderError() : React.createElement(
          Container$2,
          { textAlign: 'center' },
          React.createElement(
            'strong',
            null,
            'Oh no!'
          ),
          React.createElement(
            Box,
            { mt: 1, mb: 2 },
            error
          ),
          React.createElement(
            Button,
            { size: 'xs', variant: 'primary', outline: true, onClick: onReload },
            'Try Again'
          )
        );
      }

      return typeof children === 'function' ? children() : children;
    }
  }]);
  return Loadable;
}(React.Component);

Loadable.propTypes = {
  loading: PropTypes.bool,
  error: PropTypes.string,
  delay: PropTypes.number,
  renderLoading: PropTypes.func,
  renderError: PropTypes.func,
  onReload: PropTypes.func
};
Loadable.defaultProps = {
  loading: false,
  delay: 250
};

var StyledRadioGroup = createComponent({
  name: 'RadioGroup',
  as: Box
});

var RadioGroup = function (_Component) {
  inherits(RadioGroup, _Component);

  function RadioGroup() {
    var _ref;

    var _temp, _this, _ret;

    classCallCheck(this, RadioGroup);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = RadioGroup.__proto__ || Object.getPrototypeOf(RadioGroup)).call.apply(_ref, [this].concat(args))), _this), _this.state = {
      value: _this.props.value || null
    }, _this.handleChange = function (field, value) {
      // Bail out if value is the same
      if (_this.state.value === value) return;

      _this.setState({ value: value });

      _this.props.onChange(_this.props.name, value);
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(RadioGroup, [{
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      if (this.props.value !== undefined && this.props.value !== this.state.value) {
        this.handleChange(null, this.props.value);
      }
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      var _props = this.props,
          choices = _props.choices,
          error = _props.error,
          horizontal = _props.horizontal,
          label = _props.label,
          name = _props.name,
          color$$1 = _props.color,
          size = _props.size;


      return React.createElement(
        StyledRadioGroup,
        null,
        label && React.createElement(
          Label,
          null,
          label
        ),
        React.createElement(
          Flex,
          { flexDirection: horizontal ? 'row' : 'column' },
          choices.length && choices.map(function (choice) {
            var _choice$value = choice.value,
                value = _choice$value === undefined ? choice.id : _choice$value,
                choiceLabel = choice.label;

            var key = 'RadioButton-' + name + '-' + value;

            return React.createElement(Checkbox, {
              id: key,
              key: key,
              name: key,
              horizontal: horizontal,
              size: size,
              color: color$$1,
              label: choiceLabel,
              value: _this2.state.value,
              valueTrue: value,
              iconOn: 'radiobox-marked',
              iconOff: 'radiobox-blank',
              onChange: _this2.handleChange
            });
          })
        ),
        !!error && React.createElement(
          FormError,
          null,
          error
        )
      );
    }
  }]);
  return RadioGroup;
}(Component);

RadioGroup.propTypes = {
  name: PropTypes.string,
  onChange: PropTypes.func,
  value: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  color: PropTypes.string,
  size: PropTypes.string,
  choices: PropTypes.arrayOf(PropTypes.shape({
    value: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
    label: PropTypes.string,
    disabled: PropTypes.bool
  }))
};
RadioGroup.defaultProps = {
  choices: [],
  onChange: function onChange() {},

  color: 'primary',
  size: 'md'
};

var _templateObject$i = taggedTemplateLiteral(['\n  ', '\n'], ['\n  ', '\n']),
    _templateObject2$b = taggedTemplateLiteral(['\n      flex: 0 1 auto;\n      flex-direction: ', ';\n      flex-wrap: wrap;\n      margin-right: -', 'px;\n      margin-left: -', 'px;\n    '], ['\n      flex: 0 1 auto;\n      flex-direction: ', ';\n      flex-wrap: wrap;\n      margin-right: -', 'px;\n      margin-left: -', 'px;\n    ']);

var getMargin = function getMargin(p) {
  if (p.collapse) return 0;
  return p.gutter ? p.gutter / 2 : p.theme.grid.gutter / 2;
};

var StyledRow = createComponent({
  name: 'Row',
  as: Flex
}).extend(_templateObject$i, function (p) {
  var direction = p.vertical ? 'column' : 'row';

  return css(_templateObject2$b, p.reverse ? direction + '-reverse' : direction, getMargin, getMargin);
});

var Row = function Row(_ref) {
  var children = _ref.children,
      gutter = _ref.gutter,
      reverse = _ref.reverse,
      collapse = _ref.collapse,
      vertical = _ref.vertical,
      props = objectWithoutProperties(_ref, ['children', 'gutter', 'reverse', 'collapse', 'vertical']);
  return React.createElement(
    StyledRow,
    _extends({}, props, { gutter: gutter, reverse: reverse, collapse: collapse, vertical: vertical }),
    React.Children.map(children, function (child) {
      return React.cloneElement(child, { gutter: gutter, collapse: collapse, vertical: vertical });
    })
  );
};

Row.displayName = 'Row';

Row.propTypes = {
  /**
   * Lay row out vertically
   */
  vertical: PropTypes.bool,

  /**
   * Margin between columns
   */
  gutter: PropTypes.number,

  /**
   *  Reverse the order of the columns
   */
  reverse: PropTypes.bool,

  /**
   * Collapse columns by removing gutters
   */
  collapse: PropTypes.bool
};

Row.defaultProps = {
  vertical: false,
  reverse: false,
  collapse: false
};

var _templateObject$j = taggedTemplateLiteral(['\n  ', '\n'], ['\n  ', '\n']),
    _templateObject2$c = taggedTemplateLiteral(['\n    background: white;\n    border: 1px solid ', ';\n    height: ', 'px;\n    outline: none;\n    width: 100%;\n    padding: 0.5rem;\n    position: relative;\n    border-radius: ', 'px;\n    transition: 250ms all;\n    -webkit-appearance: none;\n    font-family: inherit;\n    font-size: ', 'px;\n    vertical-align: middle;\n\n    ', ';\n\n    select {\n      position: relative;\n      z-index: 2;\n      outline: none;\n      width: 100%;\n      font-size: ', 'px;\n      background: transparent;\n      border: none;\n      -webkit-appearance: none;\n    }\n  '], ['\n    background: white;\n    border: 1px solid ', ';\n    height: ', 'px;\n    outline: none;\n    width: 100%;\n    padding: 0.5rem;\n    position: relative;\n    border-radius: ', 'px;\n    transition: 250ms all;\n    -webkit-appearance: none;\n    font-family: inherit;\n    font-size: ', 'px;\n    vertical-align: middle;\n\n    ', ';\n\n    select {\n      position: relative;\n      z-index: 2;\n      outline: none;\n      width: 100%;\n      font-size: ', 'px;\n      background: transparent;\n      border: none;\n      -webkit-appearance: none;\n    }\n  ']),
    _templateObject3$9 = taggedTemplateLiteral(['\n        color: ', ';\n        select {\n          color: ', ';\n        }\n      '], ['\n        color: ', ';\n        select {\n          color: ', ';\n        }\n      ']),
    _templateObject4$6 = taggedTemplateLiteral(['\n  position: absolute;\n  height: 100%;\n  right: 0;\n  top: 0;\n  z-index: 1;\n'], ['\n  position: absolute;\n  height: 100%;\n  right: 0;\n  top: 0;\n  z-index: 1;\n']);

var SelectContain = createComponent({
  name: 'Select',
  as: Flex
}).extend(_templateObject$j, function (_ref) {
  var size = _ref.size,
      theme = _ref.theme,
      _ref$borderRadius = _ref.borderRadius,
      borderRadius$$1 = _ref$borderRadius === undefined ? theme.radius : _ref$borderRadius;
  return css(_templateObject2$c, theme.colors.grayLight, theme.heights[size], borderRadius$$1, theme.fontSizes[size], function (props) {
    return !props.value && css(_templateObject3$9, function (p) {
      return p.theme.colors.grayMid;
    }, function (p) {
      return p.theme.colors.grayMid;
    });
  }, function (p) {
    return p.theme.fontSizes[p.size];
  });
});

var IconContain = styled(Flex)(_templateObject4$6);

function Select(_ref2) {
  var id = _ref2.id,
      _ref2$name = _ref2.name,
      name = _ref2$name === undefined ? id : _ref2$name,
      options = _ref2.options,
      placeholder = _ref2.placeholder,
      value = _ref2.value,
      error = _ref2.error,
      _onChange = _ref2.onChange,
      _onBlur = _ref2.onBlur,
      _ref2$size = _ref2.size,
      size = _ref2$size === undefined ? 'md' : _ref2$size,
      label = _ref2.label;

  return React.createElement(
    Field,
    null,
    label && React.createElement(
      Label,
      { size: size },
      label
    ),
    React.createElement(
      SelectContain,
      { value: value, size: size },
      React.createElement(
        'select',
        {
          name: name,
          id: id,
          value: value,
          onChange: function onChange(e) {
            if (typeof _onChange === 'function') _onChange(name, e.target.value);
          },
          onBlur: function onBlur(e) {
            if (typeof _onBlur === 'function') _onBlur(name, e.target.value);
          } },
        React.createElement(
          'option',
          { value: '' },
          placeholder || 'Select an option...'
        ),
        options.map(function (option) {
          return React.createElement(
            'option',
            { key: option.value, value: option.value },
            option.label
          );
        })
      ),
      React.createElement(
        IconContain,
        { mr: 2, alignItems: 'center', justifyContent: 'center' },
        React.createElement(Icon, { name: 'chevron-down', size: 18 })
      )
    ),
    error && React.createElement(
      FormError,
      null,
      error
    )
  );
}

Select.propTypes = {
  id: PropTypes.string.isRequired,
  name: PropTypes.string.isRequired,
  options: PropTypes.array.isRequired,
  placeholder: PropTypes.string,
  value: PropTypes.string,
  error: PropTypes.string,
  onChange: PropTypes.func,
  onBlur: PropTypes.func,
  size: PropTypes.string,
  label: PropTypes.string
};

var _templateObject$k = taggedTemplateLiteral(['\n  position: relative;\n  display: inline-block;\n  width: ', 'px;\n  height: ', 'px;\n'], ['\n  position: relative;\n  display: inline-block;\n  width: ', 'px;\n  height: ', 'px;\n']),
    _templateObject2$d = taggedTemplateLiteral(['\n  display: none;\n'], ['\n  display: none;\n']),
    _templateObject3$a = taggedTemplateLiteral(['\n  ', ';\n'], ['\n  ', ';\n']),
    _templateObject4$7 = taggedTemplateLiteral(['\n    border-radius: 34px;\n    position: absolute;\n    cursor: pointer;\n    top: 0;\n    left: 0;\n    right: 0;\n    bottom: 0;\n    transition: 0.3s;\n    background-color: ', ';\n\n    &:before {\n      border-radius: 100%;\n      position: absolute;\n      content: \'\';\n      height: ', 'px;\n      width: ', 'px;\n      left: ', 'px;\n      bottom: ', 'px;\n      background-color: white;\n      transition: 0.3s;\n      ', ';\n    }\n  '], ['\n    border-radius: 34px;\n    position: absolute;\n    cursor: pointer;\n    top: 0;\n    left: 0;\n    right: 0;\n    bottom: 0;\n    transition: 0.3s;\n    background-color: ', ';\n\n    &:before {\n      border-radius: 100%;\n      position: absolute;\n      content: \'\';\n      height: ', 'px;\n      width: ', 'px;\n      left: ', 'px;\n      bottom: ', 'px;\n      background-color: white;\n      transition: 0.3s;\n      ', ';\n    }\n  ']),
    _templateObject5$4 = taggedTemplateLiteral(['\n          transform: translateX(', 'px);\n        '], ['\n          transform: translateX(', 'px);\n        ']);

var SwitchContain = styled.label(_templateObject$k, function (p) {
  return p.size * 2;
}, function (p) {
  return p.size + p.inset;
});

var SwitchInput = styled.input(_templateObject2$d);

var SwitchThumb = styled.span(_templateObject3$a, function (_ref) {
  var variant = _ref.variant,
      size = _ref.size,
      inset = _ref.inset,
      on = _ref.on,
      theme = _ref.theme,
      _ref$backgroundColor = _ref.backgroundColor,
      backgroundColor = _ref$backgroundColor === undefined ? theme.colors[variant] || theme.colors[theme.variants[variant]] : _ref$backgroundColor;
  return css(_templateObject4$7, on ? backgroundColor : theme.colors.grayMid, size, size, inset / 2, inset / 2, on && css(_templateObject5$4, size - inset));
});

var Switch = function (_React$Component) {
  inherits(Switch, _React$Component);

  function Switch() {
    var _ref2;

    var _temp, _this, _ret;

    classCallCheck(this, Switch);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref2 = Switch.__proto__ || Object.getPrototypeOf(Switch)).call.apply(_ref2, [this].concat(args))), _this), _this.state = {
      on: _this.props.value || false
    }, _this.handleChange = function () {
      var _this$props = _this.props,
          onChange = _this$props.onChange,
          name = _this$props.name;
      var on = _this.state.on;


      _this.setState({ on: !on }, function () {
        // only pass an update if onChange is defined
        if (typeof onChange === 'function') {
          _this.props.onChange(name, _this.state.on);
        }
      });
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(Switch, [{
    key: 'componentDidUpdate',
    value: function componentDidUpdate(prevProps) {
      // update switch state based on props passed in
      if ('value' in this.props) {
        if (this.props.value !== this.state.on && this.props.value !== prevProps.value) {
          this.update(this.props);
        }
      }
    }
  }, {
    key: 'update',
    value: function update(newProps) {
      this.setState({ on: newProps.value });
    }
  }, {
    key: 'render',
    value: function render() {
      var _props = this.props,
          id = _props.id,
          name = _props.name,
          size = _props.size,
          variant = _props.variant,
          inset = _props.inset;
      var on = this.state.on;


      return React.createElement(
        SwitchContain,
        { size: size, inset: inset },
        React.createElement(SwitchInput, { id: id, name: name, type: 'checkbox', on: on, onChange: this.handleChange }),
        React.createElement(SwitchThumb, { variant: variant, size: size, on: on, inset: inset })
      );
    }
  }]);
  return Switch;
}(React.Component);

Switch.propTypes = {
  id: PropTypes.string.isRequired,
  name: PropTypes.string.isRequired,
  value: PropTypes.bool,
  size: PropTypes.number,
  inset: PropTypes.number,
  variant: PropTypes.string
};
Switch.defaultProps = {
  variant: 'primary',
  size: 16,
  inset: 8
};

var _templateObject$l = taggedTemplateLiteral(['\n  display: flex;\n  flex-direction: ', ';\n'], ['\n  display: flex;\n  flex-direction: ', ';\n']),
    _templateObject2$e = taggedTemplateLiteral(['\n  display: flex;\n  margin: 0;\n  padding: 0;\n  list-style: none;\n  flex-direction: ', ';\n  ', ';\n'], ['\n  display: flex;\n  margin: 0;\n  padding: 0;\n  list-style: none;\n  flex-direction: ', ';\n  ', ';\n']),
    _templateObject3$b = taggedTemplateLiteral(['\n      border-bottom: 1px solid #dfe3e8;\n    '], ['\n      border-bottom: 1px solid #dfe3e8;\n    ']),
    _templateObject4$8 = taggedTemplateLiteral(['\n  display: flex;\n  margin: 0;\n  padding: 0;\n'], ['\n  display: flex;\n  margin: 0;\n  padding: 0;\n']),
    _templateObject5$5 = taggedTemplateLiteral(['\n  appearance: none;\n  margin: 0;\n  padding: 0;\n  background: none;\n  border: none;\n  font-size: 14px;\n  font-family: inherit;\n  position: relative;\n  justify-content: center;\n  cursor: pointer;\n  margin-', ': 1px;\n  margin-', ': -1px;\n  padding: ', ';\n  outline: none;\n  white-space: nowrap;\n  opacity: ', ';\n\n  &:active {\n    outline: none;\n  }\n'], ['\n  appearance: none;\n  margin: 0;\n  padding: 0;\n  background: none;\n  border: none;\n  font-size: 14px;\n  font-family: inherit;\n  position: relative;\n  justify-content: center;\n  cursor: pointer;\n  margin-', ': 1px;\n  margin-', ': -1px;\n  padding: ', ';\n  outline: none;\n  white-space: nowrap;\n  opacity: ', ';\n\n  &:active {\n    outline: none;\n  }\n']),
    _templateObject6$3 = taggedTemplateLiteral(['\n  color: ', ';\n  display: block;\n  padding: ', ';\n  border-', ': 4px solid ', ';\n\n  &::hover {\n    color: #212b36;\n    border-color: ', ';\n  }\n'], ['\n  color: ', ';\n  display: block;\n  padding: ', ';\n  border-', ': 4px solid ', ';\n\n  &::hover {\n    color: #212b36;\n    border-color: ', ';\n  }\n']),
    _templateObject7$3 = taggedTemplateLiteral(['\n  width: 100%;\n'], ['\n  width: 100%;\n']);

var TabsProvider = createComponent({
  name: 'Tabs'
}).extend(_templateObject$l, function (p) {
  return p.vertical ? 'row' : 'column';
});

var TabList = createComponent({
  name: 'TabList',
  tag: 'ul'
}).extend(_templateObject2$e, function (p) {
  return p.vertical ? 'column' : 'row';
}, function (p) {
  return !p.vertical && css(_templateObject3$b);
});

var TabListItem = createComponent({
  name: 'TabListItem',
  tag: 'li'
}).extend(_templateObject4$8);

var Tab = createComponent({
  name: 'Tab',
  tag: 'button'
}).extend(_templateObject5$5, function (p) {
  return p.vertical ? 'left' : 'top';
}, function (p) {
  return p.vertical ? 'right' : 'bottom';
}, function (p) {
  return p.vertical ? '0' : '0 12px';
}, function (p) {
  return p.disabled ? 0.5 : 1;
});

var TabTitle = createComponent({
  name: 'TabTitle',
  tag: 'span'
}).extend(_templateObject6$3, function (p) {
  return p.active ? '#212b36' : '#637381';
}, function (p) {
  return p.vertical ? '8px 12px 8px 8px' : '12px 8px';
}, function (p) {
  return p.vertical ? 'left' : 'bottom';
}, function (p) {
  return p.active ? p.theme.colors.primary : 'transparent';
}, function (p) {
  return p.active ? p.theme.colors.primary : '#d9dee3';
});

var TabContent = createComponent({
  name: 'TabContent',
  tag: 'div'
}).extend(_templateObject7$3);

var Tabs = function (_Component) {
  inherits(Tabs, _Component);

  function Tabs() {
    var _ref;

    var _temp, _this, _ret;

    classCallCheck(this, Tabs);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = Tabs.__proto__ || Object.getPrototypeOf(Tabs)).call.apply(_ref, [this].concat(args))), _this), _this.state = {
      active: _this.props.active || 0
    }, _this.handleTabClick = function (selected) {
      if (_this.props.onChange) {
        _this.props.onChange(selected);
      }

      _this.setState({
        active: selected
      });
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(Tabs, [{
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      if (this.props.active && this.props.active !== this.state.active) {
        this.setState({
          active: this.props.active
        });
      }
    }
  }, {
    key: 'renderHeadings',
    value: function renderHeadings() {
      var _this2 = this;

      return this.props.tabs.map(function (tab, i) {
        var id = tab.id || i;

        return React.createElement(
          TabListItem,
          { key: id },
          React.createElement(
            Tab,
            {
              id: id,
              disabled: tab.disabled,
              vertical: _this2.props.vertical,
              onClick: function onClick() {
                if (tab.onActive) {
                  tab.onActive();
                }
                _this2.handleTabClick(i);
              } },
            React.createElement(
              TabTitle,
              { active: _this2.state.active === i, vertical: _this2.props.vertical },
              tab.title
            )
          )
        );
      });
    }
  }, {
    key: 'renderContent',
    value: function renderContent() {
      var content = this.props.tabs[this.state.active].content;

      if (React.isValidElement(content)) {
        return content;
      }
      throw new Error('Tab content must be a valid React element.');
    }
  }, {
    key: 'render',
    value: function render() {
      return React.createElement(
        TabsProvider,
        { vertical: this.props.vertical },
        React.createElement(
          TabList,
          { vertical: this.props.vertical },
          this.renderHeadings()
        ),
        React.createElement(
          TabContent,
          null,
          this.renderContent()
        )
      );
    }
  }]);
  return Tabs;
}(Component);

Tabs.propTypes = {
  tabs: PropTypes.arrayOf(PropTypes.shape()).isRequired,
  active: PropTypes.number,
  vertical: PropTypes.bool,
  onChange: PropTypes.func
};
Tabs.defaultProps = {
  vertical: false
};

var defaultTheme = (function () {
  var overrides = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var colors = merge({
    primary: '#5759D6',
    grayLightest: '#f1f5f9',
    grayLight: '#cfd7df',
    grayMid: '#8297a4',
    grayDark: '#576574',
    red: '#c0392b',
    blue: '#2980b9',
    green: '#27ae60',
    orange: '#e67e22'
  }, overrides.colors);

  var radii = [0, 2, 4];

  var typography = {
    fontSize: 12
  };

  var variants = {
    primary: {
      backgroundColor: lighten(0.325, colors.primary),
      fontColor: colors.primary
    },
    success: {
      backgroundColor: lighten(0.45, colors.green),
      fontColor: colors.green
    },
    danger: {
      backgroundColor: lighten(0.45, colors.red),
      fontColor: colors.red
    },
    warning: {
      backgroundColor: lighten(0.45, colors.orange),
      fontColor: colors.orange
    },
    info: {
      backgroundColor: lighten(0.45, colors.blue),
      fontColor: colors.blue
    },
    inverted: {
      backgroundColor: colors.primary,
      fontColor: 'white'
    },
    gray: {
      backgroundColor: lighten(0.35, colors.grayMid),
      fontColor: colors.grayMid
    }
  };

  var heights = {
    xs: 28,
    sm: 32,
    md: 36,
    lg: 40,
    xl: 44
  };

  var fontSizes = {
    xs: 8,
    sm: 10,
    md: 12,
    lg: 14,
    xl: 16
  };

  var breakpoints = [368, 768, 1024, 1440];

  var grid = {
    containerMaxWidth: 1000,
    gutter: 16,
    columns: 12,
    sizes: {
      xs: breakpoints[0],
      sm: breakpoints[1],
      md: breakpoints[2],
      lg: breakpoints[3]
    }
  };

  return {
    classPrefix: 're',
    colors: colors,
    radius: 2,
    radii: radii,
    typography: typography,
    variants: variants,
    heights: heights,
    fontSizes: fontSizes,
    breakpoints: breakpoints,
    grid: grid
  };
});

var ThemeProvider$1 = function ThemeProvider$$1(props) {
  var theme = merge(defaultTheme(props.theme), props.theme);

  return React.createElement(
    ThemeProvider,
    { theme: theme },
    props.children
  );
};

export { Alert, Box, Button, Card, Checkbox, CheckboxGroup, Column, Container, Dropdown, Field, Flex, Formbot, FormError, FormGroup, Icon, Input, Label, Linkify, Modal, Loadable as Placeholder, RadioGroup, Row, Select, Spinner, Switch as Swtich, Tabs, ThemeProvider$1 as ThemeProvider };
